﻿using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.Report;
using WingInterfaceLibrary.Interface.DBInterface;
using JsonRpcLite.Services;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.Authentication;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.ReportTemplate;
using WingInterfaceLibrary.Result;
using System.Text;
using WingServerCommon.Config;
using WingInterfaceLibrary.DTO.Share;
using WingServerCommon.Config.Parameters;
using ReportService.Posters;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;

namespace ReportService.Service
{
    /// <summary>
    /// 报告服务
    /// </summary>
    public partial class ReportService : JsonRpcService, IReportService
    {
        private IUserDBService _userService;
        private IDeviceInfoDBService _deviceInfoDBService;
        private IAuthenticationService _authenticationService;
        private IOrganizationDBService _organizationDBService;
        private INotificationService _notificationService;
        private IRecordsInfoDBService _recordsInfoDBService;
        private IPatientInfoDBService _patientInfoDBService;
        private IReportDBService _reportDBService;
        private IAdminDBService _adminDBService;
        private IEmailService _emailService;
        private ISMSService _smsService;
        private IShareDBService _shareDBService;
        private IRemedicalService _remedicalService;
        private string _messageUrl = ConfigurationManager.GetParammeter<StringParameter>("Report", "MessageUrl").Value;
        private string _reportUrl = ConfigurationManager.GetParammeter<StringParameter>("Report", "ReportUrl").Value;
        private string _reportMessageTemplateId = ConfigurationManager.GetParammeter<StringParameter>("Report", "ReportMessageTemplateId").Value;

        /// <summary>
        /// Init service
        /// </summary>
        public override void Load(JsonRpcClientPool jsonRpcClientPool)
        {
            base.Load(jsonRpcClientPool);
            _userService = GetProxy<IUserDBService>();
            _authenticationService = GetProxy<IAuthenticationService>();
            _deviceInfoDBService = GetProxy<IDeviceInfoDBService>();
            _organizationDBService = GetProxy<IOrganizationDBService>();
            _notificationService = GetProxy<INotificationService>();
            _recordsInfoDBService = GetProxy<IRecordsInfoDBService>();
            _patientInfoDBService = GetProxy<IPatientInfoDBService>();
            _reportDBService = GetProxy<IReportDBService>();
            _adminDBService = GetProxy<IAdminDBService>();
            _emailService = GetProxy<IEmailService>();
            _smsService = GetProxy<ISMSService>();
            _shareDBService = GetProxy<IShareDBService>();
            _remedicalService = GetProxy<IRemedicalService>();
        }

        /// <summary>
        /// 查询报告模板详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> FindReportTemplateByCodeAsync(FindReportTemplateByCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.GetReportTemplateAsync(new GetReportTemplateDBRequest { ReportTemplateCode = request.ReportTemplateCode });
        }

        /// <summary>
        /// 查询报告详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportDTO> FindReportByCodeAsync(FindReportByCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var reportInfo = await _reportDBService.GetReportByCodeAsync(new GetReportByCodeDBRequest { ReportCode = request.ReportCode });
            if (reportInfo.DiagnosisLabels.Contains("VAid"))
            {
                var newReportDatasJson = await _remedicalService.GetReportElementByLanguageAsync(new GetReportElementByLanguageRequest
                {
                    ReportDatasJosn = reportInfo.ReportDatasJson,
                    Language = request.Language,
                });
                reportInfo.ReportDatasJson = newReportDatasJson;
            }
            return reportInfo;
        }

        /// <summary>
        /// 查询检查关联报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ReportDTO>> FindExamReportsAsync(FindExamReportsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = request.RecordCode });
        }

        public async Task<List<ReportDTO>> FindPatientReportsAsync(FindPatientReportsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.GetReportByPatientCodeAsync(request.PatientCode);
        }

        /// <summary>
        /// 查询报告标签配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ReportLabelDTO>> FindReportLabelsAsync(FindReportLabelsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                throw new RpcException(1001, "RecordCode empty error", "RecordCode empty error");
            }
            var record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
            if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
            {
                throw new RpcException(1001, "RecordInfo not exist error", "RecordInfo not exist error");
            }
            var usageType = ReportLabelUsageTypeEnum.People;
            var device = await _deviceInfoDBService.FindDeviceInfoByCodeAsync(record.DeviceCode);
            if (device != null && !string.IsNullOrWhiteSpace(device.OrganizationCode))
            {
                var organzation = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = record.OrganizationCode });
                if (organzation != null)
                {
                    usageType = (ReportLabelUsageTypeEnum)organzation.PatientType;
                }
            }
            return await _reportDBService.GetReportLabelsAsync(new GetReportLabelsDBRequest { LanguageCode = request.LanguageCode, UsageType = usageType });
        }

        /// <summary>
        /// 根据分享短码查询分享内容
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ShareDTO> FindShareContentAsync(FindShareContentRequest request)
        {
            return await _shareDBService.GetShareInfoAsync(new GetShareInfoDBRequest { ShortCode = request.ShortCode });
        }

        /// <summary>
        /// 创建报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddReportAsync(AddReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                throw new RpcException(1001, "RecordCode empty error", "RecordCode empty error");
            }
            var record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
            if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
            {
                throw new RpcException(1001, "RecordInfo not exist error", "RecordInfo not exist error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateJson))
            {
                throw new RpcException(1001, "ReportTemplateJson empty error", "ReportTemplateJson empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                throw new RpcException(1001, "ReportDatasJson empty error", "ReportDatasJson empty error");
            }

            var result = await _reportDBService.SaveReportAsync(new SaveReportDBRequest
            {
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = record.DeviceCode,
                PatientCode = record.PatientCode,
                ReportTemplateJson = request.ReportTemplateJson,
                ReportDatasJson = request.ReportDatasJson,
                PatientId = GetPatientId(request.ReportDatasJson),
                PatientName = request.PatientName,
                ReportLabels = request.ReportLabels ?? new List<string>(),
                PlatformLabels = request.PlatformLabels ?? new List<string>(),
                DiagnosisLabels = request.DiagnosisLabels ?? new List<string>(),
                ReportOrgan = request.ReportOrgan,
            });
            if (result && record.RecordStatus != RecordStatusEnum.Completed)
            {
                record.RecordStatus = RecordStatusEnum.Completed;
                await _recordsInfoDBService.UpdateRecordInfoByCodeAsync(record.RecordCode, record, null);
            }
            return result;
        }

        /// <summary>
        /// 修改报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ModifyReportAsync(ModifyReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                throw new RpcException(1001, "ReportCode empty error", "ReportCode empty error");
            }
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                throw new RpcException(1001, "RecordCode empty error", "RecordCode empty error");
            }
            var record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
            if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
            {
                throw new RpcException(1001, "RecordInfo not exist error", "RecordInfo not exist error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateJson))
            {
                throw new RpcException(1001, "ReportTemplateJson empty error", "ReportTemplateJson empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                throw new RpcException(1001, "ReportDatasJson empty error", "ReportDatasJson empty error");
            }

            var result = await _reportDBService.SaveReportAsync(new SaveReportDBRequest
            {
                ReportCode = request.ReportCode,
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = record.DeviceCode,
                PatientCode = record.PatientCode,
                ReportTemplateJson = request.ReportTemplateJson,
                ReportDatasJson = request.ReportDatasJson,
                PatientId = GetPatientId(request.ReportDatasJson),
                PatientName = request.PatientName,
                ReportLabels = request.ReportLabels ?? new List<string>(),
                PlatformLabels = request.PlatformLabels ?? new List<string>(),
                DiagnosisLabels = request.DiagnosisLabels ?? new List<string>(),
                ReportOrgan = request.ReportOrgan,
            });
            return result;
        }

        /// <summary>
        /// 修改报告标签
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ModifyReportLabelsAsync(ModifyReportLabelsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                throw new RpcException(1001, "ReportCode empty error", "ReportCode empty error");
            }

            var result = await _reportDBService.SaveReportLabelsAsync(new SaveReportLabelsDBRequest
            {
                ReportCode = request.ReportCode,
                ReportLabels = request.ReportLabels ?? new List<string>()
            });
            return result;
        }

        /// <summary>
        /// 分享报告-短信
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ShareReportBySmsAsync(ShareReportBySmsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                throw new RpcException(1001, "ReportCode empty error", "ReportCode empty error");
            }
            if (request.MobileList?.Any() != true)
            {
                throw new RpcException(1001, "MobileList empty error", "MobileList empty error");
            }
            if (string.IsNullOrWhiteSpace(_messageUrl))
            {
                throw new RpcException(1001, "Message Url empty error", "Message Url empty error");
            }
            var reportUrl = string.Format(_reportUrl, request.ReportCode);
            if (string.IsNullOrWhiteSpace(reportUrl))
            {
                throw new RpcException(1001, "Report Url empty error", "Report Url empty error");
            }
            var shortCode = await _shareDBService.AddShareInfoAsync(new AddShareInfoDBRequest { Data = new ShareDTO { ShareContent = reportUrl } });
            if (string.IsNullOrWhiteSpace(shortCode))
            {
                throw new RpcException(1001, "ShareCode empty error", "ShareCode empty error");
            }

            return await _smsService.GeneralMessageAsync(request.MobileList, _reportMessageTemplateId, new List<string> { shortCode });
        }

        /// <summary>
        /// 分享报告-邮件
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ShareReportByEmailAsync(ShareReportByEmailRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                throw new RpcException(1001, "ReportCode empty error", "ReportCode empty error");
            }
            if (request.EmailList?.Any() != true)
            {
                throw new RpcException(1001, "EmailList empty error", "EmailList empty error");
            }
            if (string.IsNullOrWhiteSpace(_messageUrl))
            {
                throw new RpcException(1001, "Message Url empty error", "Message Url empty error");
            }
            var reportUrl = string.Format(_reportUrl, request.ReportCode);
            if (string.IsNullOrWhiteSpace(reportUrl))
            {
                throw new RpcException(1001, "Report Url empty error", "Report Url empty error");
            }
            var shortCode = await _shareDBService.AddShareInfoAsync(new AddShareInfoDBRequest { Data = new ShareDTO { ShareContent = reportUrl } });
            if (string.IsNullOrWhiteSpace(shortCode))
            {
                throw new RpcException(1001, "ShareCode empty error", "ShareCode empty error");
            }

            return await _emailService.SendEmailAsync(request.EmailList, "ShareReport", GetBase64EmailString("杏聆荟|超声报告"), _messageUrl, shortCode);
        }

        /// <summary>
        /// 获取病人id
        /// </summary>
        /// <param name="reportDatasJson"></param>
        /// <returns></returns>
        private string GetPatientId(string reportDatasJson)
        {
            return string.Empty;
        }

        /// <summary>
        /// 获取邮箱Base64字符串
        /// </summary>
        /// <param name="content"></param>
        /// <returns></returns>
        private string GetBase64EmailString(string content)
        {
            content = Convert.ToBase64String(Encoding.UTF8.GetBytes(content));
            var str = $"=?UTF8?B?{content}?=";
            return str;
        }

        /// <summary>
        /// 客户端用户验证
        /// </summary>
        /// <param name="request"></param>
        /// <returns>User Code</returns>
        private async Task<string> ValidateTokenAsync(string token)
        {
            var request = new ValidateTokenRequest() { Token = token };
            var result = await _authenticationService.ValidateTokenAsync(request);
            //Check 权限
            if (result != null && result.Code != CustomerRpcCode.Ok)
            {
                throw new RpcException(1002, "Permission validation error", "Permission validation error");
            }
            return result.Token.ClientId;
        }

        /// <summary>
        /// 拷贝成个人报告模板
        /// </summary>
        /// <returns></returns>
        public async Task<bool> CopyPersonalReportTemplateAsync(CopyReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.CopyReportTemplateAsync(request.ReportTemplateCode, userCode, string.Empty);
        }

        /// <summary>
        /// 拷贝成机构报告模板
        /// </summary>
        /// <returns></returns>
        public async Task<bool> CopyOrgReportTemplateAsync(CopyOrgReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            return await _reportDBService.CopyReportTemplateAsync(request.ReportTemplateCode, string.Empty, userInfo.RootOrganizationCode);
        }

        /// <summary>
        /// 获取报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> GetReportTemplatePageAsync(FindReportTemplatePagesRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            FindReportTemplatePagesDBRequest dBRequest = new FindReportTemplatePagesDBRequest();
            dBRequest.PersonalCode = userCode;
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            dBRequest.OrganizationCode = userInfo.OrganizationCode;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.OrganizationCode });
            dBRequest.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            return await _reportDBService.GetReportTemplatePageAsync(dBRequest);
        }

        /// <summary>
        /// 获取机构报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> GetOrgReportTemplatePageAsync(FindReportTemplatePagesRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            FindReportTemplatePagesDBRequest dBRequest = new FindReportTemplatePagesDBRequest();
            dBRequest.PersonalCode = userCode;
            dBRequest.OrganizationCode = userInfo.OrganizationCode;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.OrganizationCode });
            dBRequest.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            return await _reportDBService.GetOrgReportTemplatePageAsync(dBRequest);
        }

        /// <summary>
        /// 添加报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddReportTemplateAsync(AddReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            try
            {
                var json = JsonHelper.DeserializerJson<object>(request.ReportTemplateJson);
            }
            catch (Exception ex)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "Json format error ", "Json format error");
            }
            var dbRequest = new AddReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.IsDefault = false;
            dbRequest.Data.PersonalCode = userCode;
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson;
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo == null)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "The user is not assigned an organization", "The user is not assigned an organization");
            }
            dbRequest.Data.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ReportTemplateType = ReportTemplateTypeEnum.Personal;
            return await _reportDBService.AddReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 更新报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateReportTemplateAsync(UpdateReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new UpdateReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateCode = request.ReportTemplateCode;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson;
            return await _reportDBService.UpdateReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 删除报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveReportTemplateAsync(RemoveReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportTemplateCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "ReportTemplate error", "ReportTemplate error");
            }
            var ReportTemplateInfo = await _reportDBService.GetReportTemplateByCodeAsync(request.ReportTemplateCode);
            if (ReportTemplateInfo.IsDefault)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new RemoveReportTemplateDBRequest();
            dbRequest.ReportTemplateCode = request.ReportTemplateCode;
            return await _reportDBService.RemoveReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 添加机构报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddOrgReportTemplateAsync(AddReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new AddReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.IsDefault = false;
            dbRequest.Data.OrganizationCode = userInfo.RootOrganizationCode;
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "The user is not assigned an organization", "The user is not assigned an organization");
            }
            dbRequest.Data.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ReportTemplateType = ReportTemplateTypeEnum.Organization;
            return await _reportDBService.AddReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 删除机构报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveOrgReportTemplateAsync(RemoveReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportTemplateCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "ReportTemplate error", "ReportTemplate error");
            }

            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var ReportTemplateInfo = await _reportDBService.GetReportTemplateByCodeAsync(request.ReportTemplateCode);
            if (!userInfo.IsDirector)
            {
                throw new RpcException(1002, "User not director", "User not director");
            }
            if (ReportTemplateInfo.IsDefault)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            if (ReportTemplateInfo.OrganizationCode != userInfo.OrganizationCode && ReportTemplateInfo.OrganizationCode != userInfo.RootOrganizationCode)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new RemoveReportTemplateDBRequest();
            dbRequest.ReportTemplateCode = request.ReportTemplateCode;
            return await _reportDBService.RemoveReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusAsync(AddThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new AddThesaurusDBRequest();
            dbRequest.Data = new ThesaurusDTO();
            dbRequest.Data.IsDefault = false;
            dbRequest.Data.PersonalCode = userCode;
            dbRequest.Data.ThesaurusName = request.ThesaurusName;
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo == null)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "The user is not assigned an organization", "The user is not assigned an organization");
            }
            dbRequest.Data.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ThesaurusType = ThesaurusTypeEnum.Personal;
            return await _reportDBService.AddThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 添加机构词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddOrgThesaurusAsync(AddThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo == null || !userInfo.IsDirector)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new AddThesaurusDBRequest();
            dbRequest.Data = new ThesaurusDTO();
            dbRequest.Data.IsDefault = false;
            dbRequest.Data.OrganizationCode = userInfo.RootOrganizationCode;
            dbRequest.Data.ThesaurusName = request.ThesaurusName;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "The user is not assigned an organization", "The user is not assigned an organization");
            }
            dbRequest.Data.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ThesaurusType = ThesaurusTypeEnum.Organization;
            return await _reportDBService.AddThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 更新词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusAsync(UpdateThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new UpdateThesaurusDBRequest();
            dbRequest.Data = new ThesaurusDTO();
            dbRequest.Data.ThesaurusName = request.ThesaurusName;
            dbRequest.Data.ThesaurusCode = request.ThesaurusCode;
            return await _reportDBService.UpdateThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusClassifyAsync(AddThesaurusClassifyRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusItemName))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "ThesaurusItemName error", "ThesaurusItemName error");
            }
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "Thesaurus error", "Thesaurus error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new AddThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemType = ThesaurusItemTypeEnum.Classification,
                ThesaurusItemName = request.ThesaurusItemName,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.AddThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusContentAsync(AddThesaurusContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "Thesaurus error", "Thesaurus error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new AddThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemType = ThesaurusItemTypeEnum.Contents,
                ThesaurusItemConclusion = request.ThesaurusItemConclusion ?? string.Empty,
                ThesaurusItemDescription = request.ThesaurusItemDescription ?? string.Empty,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.AddThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusClassifyAsync(UpdateThesaurusClassifyRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusItemName))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "ThesaurusItemName error", "ThesaurusItemName error");
            }
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "Thesaurus error", "Thesaurus error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new UpdateThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemCode = request.ThesaurusItemCode,
                ThesaurusItemName = request.ThesaurusItemName,
                ThesaurusItemType = ThesaurusItemTypeEnum.Classification,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.UpdateThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusContentAsync(UpdateThesaurusContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "Thesaurus error", "Thesaurus error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new UpdateThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemCode = request.ThesaurusItemCode,
                ThesaurusItemConclusion = request.ThesaurusItemConclusion ?? string.Empty,
                ThesaurusItemDescription = request.ThesaurusItemDescription ?? string.Empty,
                ThesaurusItemType = ThesaurusItemTypeEnum.Contents,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.UpdateThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 删除词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveThesaurusAsync(RemoveThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "Thesaurus error", "Thesaurus error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new RemoveThesaurusDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            return await _reportDBService.RemoveThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 删除词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveThesaurusItemAsync(RemoveThesaurusItemRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "Thesaurus error", "Thesaurus error");
            }
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (thesaurusInfo.IsDefault)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new RemoveThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItemCode = request.ThesaurusItemCode;
            return await _reportDBService.RemoveThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 更新词条库用户引用
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusUserCodeAsync(UpdateThesaurusUserCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "Thesaurus error", "Thesaurus error");
            }

            return await _reportDBService.UpdateThesaurusUserCodeAsync(
                new UpdateThesaurusUserCodeDBRequest
                {
                    UserCode = userCode,
                    ThesaurusCode = request.ThesaurusCode
                }
            );
        }

        /// <summary>
        /// 删除词条库用户引用
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteThesaurusUserCodeAsync(DeleteThesaurusUserCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "Thesaurus error", "Thesaurus error");
            }

            return await _reportDBService.DeleteThesaurusUserCodeAsync(
                new DeleteThesaurusUserCodeDBRequest
                {
                    UserCode = userCode,
                    ThesaurusCode = request.ThesaurusCode
                }
            );
        }

        /// <summary>
        /// 获取词条库（分页）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> GetThesaurusPageAsync(GetThesaurusPageRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            request.PersonalCode = userCode;
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            request.OrganizationCode = userInfo.OrganizationCode;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.OrganizationCode });
            request.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            return await _reportDBService.GetThesaurusPageAsync(request);
        }

        /// <summary>
        /// 获取词条项列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusItemsAsync(GetThesaurusItemsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.GetThesaurusItemsAsync(request);
        }

        /// <summary>
        /// 获取词条项详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ThesaurusItemDTO> GetThesaurusItemByCodeAsync(GetThesaurusItemByCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.GetThesaurusItemByCodeAsync(request);
        }


        /// <summary>
        /// 根据父节点编码获取词条分类
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusClassifyByParentCodeAsync(GetThesaurusClassifyByParentCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.GetThesaurusItemsDetailListAsync(request.ThesaurusCode, request.ParentItemCode, request.Keyword, ThesaurusItemTypeEnum.Classification);
        }

        /// <summary>
        /// 根据父节点编码获取词条内容
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusContentByParentCodeAsync(GetThesaurusContentByParentCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.GetThesaurusItemsDetailListAsync(request.ThesaurusCode, request.ParentItemCode, request.Keyword, ThesaurusItemTypeEnum.Contents);
        }

        /// <summary>
        /// 删除机构词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveOrgThesaurusAsync(RemoveThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                throw new RpcException((int)ErorrCodeEnum.ParameterError, "Thesaurus error", "Thesaurus error");
            }

            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (!userInfo.IsDirector)
            {
                throw new RpcException(1002, "User not director", "User not director");
            }
            if (thesaurusInfo.IsDefault)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            if (thesaurusInfo.OrganizationCode != userInfo.OrganizationCode && thesaurusInfo.OrganizationCode != userInfo.RootOrganizationCode)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            var dbRequest = new RemoveThesaurusDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            return await _reportDBService.RemoveThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 获取机构词条库分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> GetOrgThesaurusPageAsync(GetThesaurusPageRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            request.PersonalCode = userCode;
            request.OrganizationCode = userInfo.OrganizationCode;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.OrganizationCode });
            request.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            return await _reportDBService.GetOrgThesaurusPageAsync(request);
        }

        /// <summary>
        /// 拷贝成个人词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CopyPersonalThesaurusAsync(CopyThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.CopyThesaurusAsync(request.ThesaurusCode, userCode, string.Empty);
        }

        /// <summary>
        /// 拷贝成机构词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CopyOrgThesaurusAsync(CopyOrgThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                throw new RpcException((int)ErorrCodeEnum.BasicErorr, "No permission", "No permission");
            }
            return await _reportDBService.CopyThesaurusAsync(request.ThesaurusCode, string.Empty, userInfo.RootOrganizationCode);
        }
    }
}

